"""IoT Database Models - SQLAlchemy models for IoT sensor data storage."""

from .alert import AlertDefinition
from .base import AlertBase, AlertLevels, AlertTypes, MeasurementBase, Types, Units
from .electricity import (
    DailyElectricityUsage,
    ElectricityMeasurement,
    MonthlyElectricityUsage,
)
from .heat import DailyHeatUsage, HeatMeasurement, MonthlyHeatUsage
from .raw import RawMeasurement
from .temperature import TemperatureMeasurement
from .water import DailyWaterUsage, MonthlyWaterUsage, WaterMeasurement

__all__ = [
    "AlertDefinition",
    "AlertLevels",
    "AlertTypes",
    "DailyElectricityUsage",
    "DailyHeatUsage",
    "DailyWaterUsage",
    "ElectricityMeasurement",
    "HeatMeasurement",
    "map_daily_usage_class",
    "map_measurement_class",
    "map_monthly_usage_class",
    "MonthlyElectricityUsage",
    "MonthlyHeatUsage",
    "MonthlyWaterUsage",
    "RawMeasurement",
    "TemperatureMeasurement",
    "Types",
    "Units",
    "WaterMeasurement",
]


def map_measurement_class(type_: Types):
    match type_:
        case Types.electricity:
            return ElectricityMeasurement
        case Types.water:
            return WaterMeasurement
        case Types.heat:
            return HeatMeasurement
        case Types.temperature:
            return TemperatureMeasurement
        case _:
            raise NotImplementedError


def map_daily_usage_class(type_: Types):
    match type_:
        case Types.electricity:
            return DailyElectricityUsage
        case Types.water:
            return DailyWaterUsage
        case Types.heat:
            return DailyHeatUsage
        case Types.temperature:
            return None
        case _:
            raise NotImplementedError


def map_monthly_usage_class(type_: Types):
    match type_:
        case Types.electricity:
            return MonthlyElectricityUsage
        case Types.water:
            return MonthlyWaterUsage
        case Types.heat:
            return MonthlyHeatUsage
        case Types.temperature:
            return None
        case _:
            raise NotImplementedError



from sqlalchemy import Column, UniqueConstraint
from sqlalchemy.dialects.postgresql import JSON

from .base import AlertBase


class AlertDefinition(AlertBase):
    __tablename__ = "alert_definition"
    __table_args__ = (
        UniqueConstraint(
            "tenant",
            "meter_id",
        ),
    )

    properties = Column(JSON)



"""Init schema

Revision ID: 78371c256762
Revises: 
Create Date: 2024-08-21 00:35:06.960907

"""

from typing import Sequence, Union

import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "78371c256762"
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "measurement_electricity",
        sa.Column("measured_ts", sa.DateTime(), nullable=True),
        sa.Column("value", sa.Float(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id", "measured_ts"),
    )
    op.create_table(
        "measurement_heat",
        sa.Column("measured_ts", sa.DateTime(), nullable=True),
        sa.Column("value", sa.Float(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id", "measured_ts"),
    )
    op.create_table(
        "measurement_raw",
        sa.Column("data", postgresql.JSON(astext_type=sa.Text()), nullable=True),
        sa.Column("type", sa.SmallInteger(), nullable=True),
        sa.Column("is_processed", sa.Boolean(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "measurement_water",
        sa.Column("measured_ts", sa.DateTime(), nullable=True),
        sa.Column("value", sa.Float(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id", "measured_ts"),
    )
    op.create_table(
        "usage_electricity_daily",
        sa.Column("start_ts", sa.DateTime(), nullable=True),
        sa.Column("end_ts", sa.DateTime(), nullable=True),
        sa.Column("consumption_ts", sa.DateTime(), nullable=True),
        sa.Column("start_value", sa.Float(), nullable=True),
        sa.Column("end_value", sa.Float(), nullable=True),
        sa.Column("consumption", sa.Float(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id", "consumption_ts"),
    )
    op.create_table(
        "usage_electricity_monthly",
        sa.Column("start_ts", sa.DateTime(), nullable=True),
        sa.Column("end_ts", sa.DateTime(), nullable=True),
        sa.Column("consumption_ts", sa.DateTime(), nullable=True),
        sa.Column("start_value", sa.Float(), nullable=True),
        sa.Column("end_value", sa.Float(), nullable=True),
        sa.Column("consumption", sa.Float(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id", "consumption_ts"),
    )
    op.create_table(
        "usage_heat_daily",
        sa.Column("start_ts", sa.DateTime(), nullable=True),
        sa.Column("end_ts", sa.DateTime(), nullable=True),
        sa.Column("consumption_ts", sa.DateTime(), nullable=True),
        sa.Column("start_value", sa.Float(), nullable=True),
        sa.Column("end_value", sa.Float(), nullable=True),
        sa.Column("consumption", sa.Float(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id", "consumption_ts"),
    )
    op.create_table(
        "usage_heat_monthly",
        sa.Column("start_ts", sa.DateTime(), nullable=True),
        sa.Column("end_ts", sa.DateTime(), nullable=True),
        sa.Column("consumption_ts", sa.DateTime(), nullable=True),
        sa.Column("start_value", sa.Float(), nullable=True),
        sa.Column("end_value", sa.Float(), nullable=True),
        sa.Column("consumption", sa.Float(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id", "consumption_ts"),
    )
    op.create_table(
        "usage_water_daily",
        sa.Column("start_ts", sa.DateTime(), nullable=True),
        sa.Column("end_ts", sa.DateTime(), nullable=True),
        sa.Column("consumption_ts", sa.DateTime(), nullable=True),
        sa.Column("start_value", sa.Float(), nullable=True),
        sa.Column("end_value", sa.Float(), nullable=True),
        sa.Column("consumption", sa.Float(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id", "consumption_ts"),
    )
    op.create_table(
        "usage_water_monthly",
        sa.Column("start_ts", sa.DateTime(), nullable=True),
        sa.Column("end_ts", sa.DateTime(), nullable=True),
        sa.Column("consumption_ts", sa.DateTime(), nullable=True),
        sa.Column("start_value", sa.Float(), nullable=True),
        sa.Column("end_value", sa.Float(), nullable=True),
        sa.Column("consumption", sa.Float(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id", "consumption_ts"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("usage_water_monthly")
    op.drop_table("usage_water_daily")
    op.drop_table("usage_heat_monthly")
    op.drop_table("usage_heat_daily")
    op.drop_table("usage_electricity_monthly")
    op.drop_table("usage_electricity_daily")
    op.drop_table("measurement_water")
    op.drop_table("measurement_raw")
    op.drop_table("measurement_heat")
    op.drop_table("measurement_electricity")
    # ### end Alembic commands ###



"""Create alerts models

Revision ID: 0d7bea8f4d60
Revises: fc391d505594
Create Date: 2025-08-04 22:33:08.875952

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision: str = '0d7bea8f4d60'
down_revision: Union[str, None] = 'fc391d505594'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('alert_definition',
    sa.Column('properties', postgresql.JSON(astext_type=sa.Text()), nullable=True),
    sa.Column('type', sa.SmallInteger(), nullable=True),
    sa.Column('level', sa.SmallInteger(), nullable=True),
    sa.Column('device_name', sa.Text(), nullable=True),
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('tenant', sa.UUID(), nullable=True),
    sa.Column('external_id', sa.Text(), nullable=True),
    sa.Column('meter_id', sa.Text(), nullable=True),
    sa.Column('created_ts', sa.DateTime(), nullable=True),
    sa.Column('updated_ts', sa.DateTime(), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('tenant', 'meter_id')
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('alert_definition')
    # ### end Alembic commands ###



"""Add temperature

Revision ID: fc391d505594
Revises: 78371c256762
Create Date: 2025-06-15 20:03:15.039980

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "fc391d505594"
down_revision: Union[str, None] = "78371c256762"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "measurement_temperature",
        sa.Column("measured_ts", sa.DateTime(), nullable=True),
        sa.Column("value", sa.Float(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id", "measured_ts"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("measurement_temperature")
    # ### end Alembic commands ###



from sqlalchemy import UniqueConstraint

from .base import MeasurementBase, UsageBase


class WaterMeasurement(MeasurementBase):
    __tablename__ = "measurement_water"
    __table_args__ = (
        UniqueConstraint(
            "tenant",
            "meter_id",
            "measured_ts",
        ),
    )
    RAW_VALUE_ATTR_NAME = "total_m3"


class DailyWaterUsage(UsageBase):
    __tablename__ = "usage_water_daily"
    __table_args__ = (
        UniqueConstraint(
            "tenant",
            "meter_id",
            "consumption_ts",
        ),
    )


class MonthlyWaterUsage(UsageBase):
    __tablename__ = "usage_water_monthly"
    __table_args__ = (
        UniqueConstraint(
            "tenant",
            "meter_id",
            "consumption_ts",
        ),
    )



from sqlalchemy import UniqueConstraint

from .base import MeasurementBase, UsageBase


class ElectricityMeasurement(MeasurementBase):
    __tablename__ = "measurement_electricity"
    __table_args__ = (
        UniqueConstraint(
            "tenant",
            "meter_id",
            "measured_ts",
        ),
    )

    RAW_VALUE_ATTR_NAME = "total_energy_consumption_kwh"


class DailyElectricityUsage(UsageBase):
    __tablename__ = "usage_electricity_daily"
    __table_args__ = (
        UniqueConstraint(
            "tenant",
            "meter_id",
            "consumption_ts",
        ),
    )


class MonthlyElectricityUsage(UsageBase):
    __tablename__ = "usage_electricity_monthly"
    __table_args__ = (
        UniqueConstraint(
            "tenant",
            "meter_id",
            "consumption_ts",
        ),
    )



"""IoT Database Models - SQLAlchemy models for IoT sensor data storage."""

__version__ = "0.2.1"
__author__ = "NG IT Services Sp. z o.o."
__email__ = "biuro@ngits.pl"

from .models import *  # noqa: F403, F401



from datetime import datetime
from enum import Enum as PyEnum
from enum import IntEnum

from sqlalchemy import Column, DateTime, Float, Integer, SmallInteger, Text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.ext.hybrid import hybrid_property

Base = declarative_base()


class Units(PyEnum):
    m3 = "m3"
    kwh = "kwh"
    degc = "degc"


class Types(IntEnum):
    water = 1
    electricity = 2
    heat = 3
    temperature = 4


class AlertTypes(IntEnum):
    # Value-based alerts
    threshold_high = 1  # Upper threshold breach
    threshold_low = 2  # Lower threshold breach
    sudden_change = 3  # Sudden value change detection

    # Data availability alerts
    offline = 10  # Device not responding
    data_gap = 11  # Gap in data transmission
    data_delayed = 12  # Delayed data reception

    # Data quality alerts
    invalid_reading = 20  # Invalid measurement detected
    stuck_value = 21  # Value not changing over extended period
    noise = 22  # Excessive fluctuations in measurements


class AlertLevels(IntEnum):
    emergency = 0  # Highest priority - immediate action required (safety threats)
    critical = 1  # Requires immediate response (e.g. 24h data loss)
    warning = 2  # Requires attention (e.g. data delay > 1h)
    info = 3  # Informational (minor deviations from norm)


class TimeSign(Base):
    __abstract__ = True

    created_ts = Column(DateTime, default=datetime.utcnow)
    updated_ts = Column(DateTime, default=datetime.utcnow, onupdate=datetime.utcnow)


class IdentityBase(Base):
    __abstract__ = True

    id = Column(Integer, primary_key=True)
    tenant = Column(UUID(as_uuid=True))
    external_id = Column(Text, nullable=True)
    meter_id = Column(Text)


class MeasurementBase(IdentityBase, TimeSign, Base):
    __abstract__ = True

    measured_ts = Column(DateTime)
    value = Column(Float)


class UsageBase(IdentityBase, TimeSign, Base):
    __abstract__ = True

    start_ts = Column(DateTime)
    end_ts = Column(DateTime)
    consumption_ts = Column(DateTime)

    start_value = Column(Float)
    end_value = Column(Float)
    consumption = Column(Float)


class AlertBase(IdentityBase, TimeSign, Base):
    __abstract__ = True

    _type = Column("type", SmallInteger)
    _level = Column("level", SmallInteger)
    device_name = Column(Text)

    @hybrid_property
    def type(self):
        return AlertTypes(self._type) if self._type is not None else None

    @type.setter
    def type(self, value):
        self._type = value.value if value is not None else None

    @hybrid_property
    def level(self):
        return AlertLevels(self._level) if self._level is not None else None

    @level.setter
    def level(self, value):
        self._level = value.value if value is not None else None



from sqlalchemy import Boolean, Column, SmallInteger
from sqlalchemy.dialects.postgresql import JSON

from .base import Base, IdentityBase, TimeSign, Types


class RawMeasurement(IdentityBase, TimeSign, Base):
    __tablename__ = "measurement_raw"

    data = Column(JSON)
    type = Column(SmallInteger)
    is_processed = Column(Boolean, default=False)

    @property
    def type_enum(self):
        return Types(self.type)

    @type_enum.setter
    def type_enum(self, type_enum):
        self.type = type_enum.value
